@@ -28,7 +28,7 @@
 #include "SDL_version.h"
 
 
-#define DEFAULT_ARCHIVERS_ALLOCATE 1
+#define DEFAULT_ARCHIVERS_ALLOCATE 16
 
 
 typedef struct _Archiver
@@ -271,37 +271,6 @@ void DirContainer_Free(DirContainer *dirContainer)
 	int i;
 	
 	for (i = 0; i < dirContainer->dirNum; i++) {
-#ifdef DEBUG
-		int j;
-		PieceOfDirectory *dir;
-		dir = &dirContainer->piecesOfDir[i];
-		printf("%d:", i);
-		printf("%s", dir->dirName);
-		printf("(parent:%d)", dir->parentDirIndex);
-		if (dir->childDirIndexNum > 0) {
-			if (dir->childDirIndexNum == 1) {
-				printf("(child");
-			} else {
-				printf("(children");
-			}
-			for (j = 0; j < dir->childDirIndexNum; j++) {
-				printf(":%d", dir->childDirIndex[j]);
-			}
-			printf(")");
-		}
-		if (dir->fileIndexNum > 0) {
-			if (dir->fileIndexNum == 1) {
-				printf("(file");
-			} else {
-				printf("(files");
-			}
-			for (j = 0; j < dir->fileIndexNum; j++) {
-				printf(":%d", dir->fileIndex[j]);
-			}
-			printf(")");
-		}
-		printf("\n");
-#endif
 		free(dirContainer->piecesOfDir[i].dirName);
 		free(dirContainer->piecesOfDir[i].childDirIndex);
 		free(dirContainer->piecesOfDir[i].fileIndex);
@@ -518,21 +487,76 @@ int DirContainer_ReallocCurrentPath(DirContainer *dirContainer, int length)
 }
 
 
+int DirContainer_(
+	DirContainer *dirContainer, const char *path, int flags,
+	int *nowDirIndex, int *readLength)
+{
+	char *dirName;
+	int dirNameLength;
+	int i;
+	int absoluteIndex;
+	char *compareDirName;
+	int childDirIndexNum;
+	int result;
+	
+	dirNameLength = 0;
+	while (path[dirNameLength] != '\0' && path[dirNameLength] != '/') {
+		dirNameLength++;
+	}
+	*readLength = dirNameLength;
+	if (dirNameLength == 0) {
+		return ARCHIVE_ERROR_INVALID_NAME;
+	}
+	dirName = (char*)malloc(dirNameLength + 1);
+	if (dirName == NULL) {
+		return ARCHIVE_ERROR_ALLOCATE;
+	}
+	strncpy(dirName, path, dirNameLength);
+	dirName[dirNameLength] = '\0';
+	childDirIndexNum =
+		dirContainer->piecesOfDir[*nowDirIndex].childDirIndexNum;
+	for (i = 0; i < childDirIndexNum; i++) {
+		absoluteIndex =
+			dirContainer->piecesOfDir[*nowDirIndex].childDirIndex[i];
+		compareDirName =
+			dirContainer->piecesOfDir[absoluteIndex].dirName;
+		if (strcmp(dirName, compareDirName) == 0) {
+			break;
+		}
+	}
+	if (i >= dirContainer->piecesOfDir[*nowDirIndex].childDirIndexNum) {
+		/* not found */
+		if (flags & DIRECTORY_ADD_DIR) {
+			result = DirContainer_AddDirectoryToDirIndex(
+				dirContainer, dirName, *nowDirIndex);
+			if (result != ARCHIVE_SUCCESS) {
+				free(dirName);
+				return result;
+			}
+		} else {
+			free(dirName);
+			return ARCHIVE_ERROR_NO_EXIST_DIRECTORY;
+		}
+	}
+	free(dirName);
+	*nowDirIndex =
+		dirContainer->piecesOfDir[*nowDirIndex].childDirIndex[i];
+	
+	return ARCHIVE_SUCCESS;
+}
+
+
 /* path delimiter is '/' */
 int DirContainer_ParsePath(
 	DirContainer *dirContainer, const char *path,
 	int flags, int *returnDirIndex)
 {
-	char *dirName;
-	int dirNameLength;
 	int position;
 	int lastDirNamePosition;
 	int nowDirIndex;
-	int i;
 	int result;
-	int absoluteIndex;
-	char *compareDirName;
 	int isFromRoot;
+	int readLength;
 	
 	if (path[0] == '/') {
 		position = 1;
@@ -546,49 +570,13 @@ int DirContainer_ParsePath(
 		isFromRoot = SDL_FALSE;
 	}
 	while (path[lastDirNamePosition] != '\0') {
-		dirNameLength = 0;
-		while (path[position] != '\0' && path[position] != '/') {
-			dirNameLength++;
-			position++;
+		result = DirContainer_(
+			dirContainer, path + lastDirNamePosition, flags,
+			&nowDirIndex, &readLength);
+		if (result != ARCHIVE_SUCCESS) {
+			return result;
 		}
-		if (dirNameLength == 0) {
-			return ARCHIVE_ERROR_INVALID_NAME;
-		}
-		dirName = (char*)malloc(dirNameLength + 1);
-		if (dirName == NULL) {
-			return ARCHIVE_ERROR_ALLOCATE;
-		}
-		strncpy(dirName, path + lastDirNamePosition, dirNameLength);
-		dirName[dirNameLength] = '\0';
-		for (
-			i = 0;
-			i < dirContainer->piecesOfDir[nowDirIndex].childDirIndexNum;
-			i++
-		) {
-			absoluteIndex =
-				dirContainer->piecesOfDir[nowDirIndex].childDirIndex[i];
-			compareDirName =
-				dirContainer->piecesOfDir[absoluteIndex].dirName;
-			if (strcmp(dirName, compareDirName) == 0) {
-				break;
-			}
-		}
-		if (i >= dirContainer->piecesOfDir[nowDirIndex].childDirIndexNum) {
-			/* not found */
-			if (flags & DIRECTORY_ADD_DIR) {
-				result = DirContainer_AddDirectoryToDirIndex(
-					dirContainer, dirName, nowDirIndex);
-				if (result != ARCHIVE_SUCCESS) {
-					free(dirName);
-					return result;
-				}
-			} else {
-				return ARCHIVE_ERROR_NO_EXIST_DIRECTORY;
-			}
-		}
-		free(dirName);
-		nowDirIndex =
-			dirContainer->piecesOfDir[nowDirIndex].childDirIndex[i];
+		position += readLength;
 		if (path[position] == '\0') {
 			break;
 		}
