@@ -32,7 +32,7 @@
 
 /* typeflag */
 #define REGTYPE  '0'   /* Regular file  */
-#define AREGTYPE '_0' /* Regular file  */
+#define AREGTYPE '\0' /* Regular file  */
 #define LNKTYPE  '1'   /* Link          */
 #define SYMTYPE  '2'   /* Reserved      */
 #define CHRTYPE  '3'   /* Character special */
@@ -40,6 +40,7 @@
 #define DIRTYPE  '5'   /* Directory     */
 #define FIFOTYPE '6'   /* FIFO special  */
 #define CONTTYPE '7'   /* Reserved      */
+#define LONGTYPE 'L'   /* LongLink */
 
 typedef struct _tar_header
 {
@@ -110,6 +111,11 @@ static int Tar_Seek(SDL_Archive *archive, const long offset, const int whence);
 static SDL_Archive *Tar_Clone(SDL_Archive *archive);
 
 static int Tar_GetFileData(TAR_DATA *tar, int fileSeek);
+static int Tar_SetPath(TAR_DATA *tar, TAR_CHAIN *newChain, char *path);
+static int Tar_ConvertOctadecimalFileSizeToDecimal(TAR_HEADER *tarHeader);
+static char *Tar_CreateFilePathByLongLink(TAR_DATA *tar, long size, int *block);
+static char *Tar_CreateFilePathByHeader(TAR_HEADER *tarHeader);
+static void Tar_FreeFilePath(char *string);
 
 
 int Archive_IsTar(const char *file)
@@ -123,7 +129,7 @@ int Archive_IsTar(const char *file)
 	}
 	fread(&tarHeader, TAR_BLOCK, 1, fp);
 	fclose(fp);
-	if (memcmp(tarHeader.version, "0", 2) != 0) {
+	if (memcmp(tarHeader.version, "00 ", 2) != 0) {
 		return SDL_FALSE;
 	}
 	
@@ -177,17 +183,134 @@ SDL_Archive *Archive_FromTar(const char *file)
 }
 
 
+int Tar_SetPath(TAR_DATA *tar, TAR_CHAIN *newChain, char *path)
+{
+	int dirIndex;
+	int result;
+	char *fileNameTemp;
+	int fileNameLength;
+	char *lastSlash;
+	
+	lastSlash = strrchr(path, '/');
+	if (lastSlash == NULL) {
+		result = DirContainer_ParsePath(
+			tar->dirContainer, "/", DIRECTORY_NON, &dirIndex);
+		if (result != ARCHIVE_SUCCESS) {
+			return result;
+		}
+		newChain->fileName = malloc(strlen(path) + 1);
+		if (newChain->fileName == NULL) {
+			return ARCHIVE_ERROR_ALLOCATE;
+		}
+		strcpy(newChain->fileName, path);
+	} else {
+		*lastSlash = '\0';
+		result = DirContainer_ParsePath(
+			tar->dirContainer, "/",
+			DIRECTORY_MOVE_DIR, NULL);
+		if (result != ARCHIVE_SUCCESS) {
+			return result;
+		}
+		result = DirContainer_ParsePath(
+			tar->dirContainer, path,
+			DIRECTORY_ADD_DIR, &dirIndex);
+		if (result != ARCHIVE_SUCCESS) {
+			return result;
+		}
+		fileNameLength = strlen(lastSlash + 1);
+		newChain->fileName = malloc(fileNameLength + 1);
+		if (newChain->fileName == NULL) {
+			return ARCHIVE_ERROR_ALLOCATE;
+		}
+		strcpy(newChain->fileName, lastSlash + 1);
+	}
+	if (strlen(path) == 0) {
+		return ARCHIVE_ERROR_INVALID_NAME;
+	}
+	DirContainer_AddFileToDirIndex(
+		tar->dirContainer, tar->chainNum, dirIndex);
+	
+	return ARCHIVE_SUCCESS;
+}
+
+
+int Tar_ConvertOctadecimalFileSizeToDecimal(TAR_HEADER *tarHeader)
+{
+	int i;
+	long size;
+	char sizetemp[2];
+	
+	size = 0L;
+	for (i = 0; i < 12; i++) {
+		sizetemp[0] = tarHeader->size[i - 1];
+		sizetemp[1] = '\0';
+		size += (long)(atoi(sizetemp) * pow(8, 11 - i));
+	}
+	
+	return size;
+}
+
+
+char *Tar_CreateFilePathByLongLink(TAR_DATA *tar, long size, int *block)
+{
+	int readBlock;
+	char *longLink;
+	
+	readBlock = size / TAR_BLOCK;
+	if (size % TAR_BLOCK != 0) {  /* odd */
+		readBlock += 1;
+	}
+	
+	longLink = malloc(size + 1);
+	if (longLink == NULL) {
+		fseek(tar->fp, readBlock * TAR_BLOCK, SEEK_CUR);
+		*block = readBlock;
+		return NULL;
+	}
+	
+	fread(longLink, size, 1, tar->fp);
+	longLink[size] = '\0';
+	
+	fseek(tar->fp, readBlock * TAR_BLOCK - size, SEEK_CUR);
+	*block = readBlock;
+	
+	return longLink;
+}
+
+
+char *Tar_CreateFilePathByHeader(TAR_HEADER *tarHeader)
+{
+	char *name;
+	
+	name = malloc(strlen(tarHeader->name) + 1);
+	if (name == NULL) {
+		return NULL;
+	}
+	
+	strcpy(name, tarHeader->name);
+	
+	return name;
+}
+
+
+void Tar_FreeFilePath(char *string)
+{
+	free(string);
+}
+
+
+/* return quantity of loaded block */
 int Tar_GetFileData(TAR_DATA *tar, int fileSeek)
 {
 	TAR_CHAIN *newChain;
-	int dirIndex;
 	void *tempptr;
-	char *fileNameTemp;
-	int fileNameLength;
-	char sizetemp[2];
 	int blockNum;
-	int i;
+	int result;
+	int longFileNameBlock;
 	TAR_HEADER tarHeader;
+	char *filePath;
+	
+	newChain = &tar->chain[tar->chainNum];
 	
 	if (tar->chainNum >= tar->chainAllocated) {
 		tar->chainAllocated = tar->chainAllocated * 2;
@@ -198,56 +321,61 @@ int Tar_GetFileData(TAR_DATA *tar, int fileSeek)
 		}
 		tar->chain = (TAR_CHAIN *)tempptr;
 	}
-	newChain = &tar->chain[tar->chainNum];
 	fread(&tarHeader, TAR_BLOCK, 1, tar->fp);
 	blockNum = 1;
-	if (tarHeader.typeflag == DIRTYPE) {
-		DirContainer_ParsePath(
-			tar->dirContainer, "/",
-			DIRECTORY_MOVE_DIR, NULL);
-		DirContainer_ParsePath(
-			tar->dirContainer, tarHeader.name,
-			DIRECTORY_ADD_DIR, NULL);
-		return blockNum;
-	}
-	fileNameTemp = strrchr(tarHeader.name, '/');
-	if (fileNameTemp == NULL) {
-		DirContainer_ParsePath(
-			tar->dirContainer, "/", DIRECTORY_NON, &dirIndex);
-		newChain->fileName = malloc(strlen(tarHeader.name) + 1);
-		if (newChain->fileName == NULL) {
+	
+	newChain->size = Tar_ConvertOctadecimalFileSizeToDecimal(&tarHeader);
+	
+	if (tarHeader.typeflag == LONGTYPE) {
+		filePath = Tar_CreateFilePathByLongLink(
+				tar, newChain->size, &longFileNameBlock);
+		if (filePath == NULL) {
+			return blockNum;
+		}
+		blockNum += longFileNameBlock;
+		
+		fread(&tarHeader, TAR_BLOCK, 1, tar->fp);
+		blockNum += 1;
+		newChain->size =
+			Tar_ConvertOctadecimalFileSizeToDecimal(&tarHeader);
+	} else if (
+		tarHeader.typeflag == REGTYPE ||
+		tarHeader.typeflag == AREGTYPE
+	) {
+		filePath = Tar_CreateFilePathByHeader(&tarHeader);
+		if (filePath == NULL) {
 			return blockNum;
 		}
-		strcpy(newChain->fileName, tarHeader.name);
 	} else {
-		*fileNameTemp = '\0';
-		DirContainer_ParsePath(
-			tar->dirContainer, "/",
-			DIRECTORY_MOVE_DIR, NULL);
-		DirContainer_ParsePath(
-			tar->dirContainer, tarHeader.name,
-			DIRECTORY_ADD_DIR, &dirIndex);
-		fileNameLength = strlen(fileNameTemp + 1);
-		newChain->fileName = malloc(fileNameLength + 1);
-		if (newChain->fileName == NULL) {
-			return blockNum;
-		}
-		strcpy(newChain->fileName, fileNameTemp + 1);
-	}
-	if (strlen(tarHeader.name) == 0) {
 		return blockNum;
 	}
-	DirContainer_AddFileToDirIndex(
-		tar->dirContainer, tar->chainNum, dirIndex);
-	newChain->size = 0L;
-	newChain->seek = fileSeek + TAR_BLOCK;
-	/* convert octadecimal to decimal */
-	for (i = 0; i < 12; i++) {
-		sizetemp[0] = tarHeader.size[i - 1];
-		sizetemp[1] = '\0';
-		newChain->size +=
-			(long)(atoi(sizetemp) * pow(8, 11 - i));
+	
+	if (tarHeader.typeflag == DIRTYPE) {
+		result = DirContainer_ParsePath(
+			tar->dirContainer, "/",
+			DIRECTORY_MOVE_DIR, NULL);
+		if (result != ARCHIVE_SUCCESS) {
+			Tar_FreeFilePath(filePath);
+			return blockNum;
+		}
+		result = DirContainer_ParsePath(
+			tar->dirContainer, filePath,
+			DIRECTORY_ADD_DIR, NULL);
+		if (result != ARCHIVE_SUCCESS) {
+			Tar_FreeFilePath(filePath);
+			return blockNum;
+		}
+		Tar_FreeFilePath(filePath);
+		return blockNum;
 	}
+	newChain->seek = fileSeek + TAR_BLOCK;
+	
+	result = Tar_SetPath(tar, newChain, filePath);
+	Tar_FreeFilePath(filePath);
+	if (result != ARCHIVE_SUCCESS) {
+		return blockNum;
+	}
+	
 	blockNum += newChain->size / TAR_BLOCK;
 	if (newChain->size % TAR_BLOCK != 0) { /* odd */
 		blockNum++;
